# Sessionless
## What it is not: A replacement for sessions
Sessionless is **not** a tool to replace sessions. For some misconceptions about this, see [here](http://cryto.net/~joepie91/blog/2016/06/13/stop-using-jwt-for-sessions/)/[here](http://cryto.net/~joepie91/blog/2016/06/19/stop-using-jwt-for-sessions-part-2-why-your-solution-doesnt-work/). (So a posteriori, i would not have named this module like i did.)
Sessionless is especially nothing for non security-experts. So if a Drupal shop installs this on your site without a thorough security audit, sue them!

## What it is: A tool to avoid sessions in some cases
Sessionless leverages encrypted and signed JsonWebTokens.

## Sessionless module
Sessionless provides
- a WebToken services, that provides encrypted and signed JsonWebTokens, including a CryptoKeyStorage in site state.
- a form element that works like the "hidden" element, but stores values signed and encrypted.

## SessionlessSession module
SessionlessSession provides an API to store a session in a sticky query.
Again, this can not in general replace sessions. Security caveats:
- The url token is not revokable
- The url token can be stolen by any third party JS and used for SessionRiding and other attacks

## SessionlessForms (unfinished, UI-disabled)
Caches form and formState in encrypted signed hidden form fields, obsoleting server side form cache.

## Note on changing crypto algorithms
If you change crypto algorithms, you must delete the stored keys:
```sh
vendor/bin/drush sdel sessionless.key.encryption
vendor/bin/drush sdel sessionless.key.signature
```

## Note on (a)symmetric crypto
Tests show that some asymmetric crypto can be really slow (like 0.2 sec  for RSA with 1024 bit keys, even 30+ sec for 4096 bit keys). While [Benchmarks](https://web-token.spomky-labs.com/v/v2.x/benchmark/result-table) suggest this can be remedied, asymmetric crypto is not necessary for any current use case. YMMV though.
