<?php

namespace Drupal\sessionless_session;

use Drupal\sessionless\WebTokenService\WebTokenEncryptAndSignService;
use Drupal\sticky_query\StickyQueryStorage\StickyQueryStorageInterface;

/**
 * StickyQueryDecorator that signs and encrypts.
 */
final class StickyQueryStorageEncryptionDecorator implements StickyQueryStorageInterface {

  protected StickyQueryStorageInterface $decorated;

  protected WebTokenEncryptAndSignService $cryptoService;

  protected bool $mustEncrypt;

  /**
   * @param \Drupal\sticky_query\StickyQueryStorage\StickyQueryStorageInterface $decorated
   * @param \Drupal\sessionless\WebTokenService\WebTokenEncryptAndSignService $cryptoService
   * @param bool $mustEncrypt
   */
  public function __construct(StickyQueryStorageInterface $decorated, WebTokenEncryptAndSignService $cryptoService, bool $mustEncrypt = TRUE) {
    $this->decorated = $decorated;
    $this->cryptoService = $cryptoService;
    $this->mustEncrypt = $mustEncrypt;
  }

  public function getValue() {
    $data = $this->decorated->getValue();
    if ($data) {
      $jwtString = $this->cryptoService->dump($data);
    }
    else {
      $jwtString = NULL;
    }
    return $jwtString;
  }

  public function setValue($value): void {
    if ($value && is_string($value)) {
      $this->decorated->setValue($this->cryptoService->parse($value));
    }
    else {
      $this->decorated->setValue(NULL);
    }
  }

}
