<?php

namespace Drupal\sessionless\KeyStorage;

use Drupal\Core\State\StateInterface;
use Drupal\sessionless\WebTokenAlgorithms\WebTokenAlgorithmsInterface;
use Jose\Component\Core\JWK;

/**
 * Key Storage
 *
 * @internal
 */
final class KeyStorage implements KeyStorageInterface {

  private const STATER_KEY_FOR_SIGNATURE_KEY = "sessionless.key.signature";

  private const STATER_KEY_FOR_ENCRYPTION_KEY = "sessionless.key.encryption";

  private const STATER_KEY_FOR_KEY_VERSION = "sessionless.key.version";

  public function __construct(
    protected WebTokenAlgorithmsInterface $tokenAlgorithms,
    protected StateInterface $state,
  ) {}

  public function getSignatureKey(): JWK {
    $key = $this->state->get(self::STATER_KEY_FOR_SIGNATURE_KEY);
    if (!$key) {
      $key = $this->tokenAlgorithms->getSignatureKey();
      $this->state->set(self::STATER_KEY_FOR_SIGNATURE_KEY, $key);
    }
    return $key;
  }

  public function getEncryptionKey(): JWK {
    $key = $this->state->get(self::STATER_KEY_FOR_ENCRYPTION_KEY);
    if (!$key) {
      $key = $this->tokenAlgorithms->getEncryptionKey();
      $this->state->set(self::STATER_KEY_FOR_ENCRYPTION_KEY, $key);
    }
    return $key;
  }

  public function dropKeys(): void {
    $this->state->delete(self::STATER_KEY_FOR_SIGNATURE_KEY);
    $this->state->delete(self::STATER_KEY_FOR_ENCRYPTION_KEY);
    // Increment key version.
    $keyVersion = $this->getKeyVersion();
    $keyVersion++;
    $this->state->set(self::STATER_KEY_FOR_KEY_VERSION, $keyVersion);
  }

  public function getKeyVersion(): int {
    return $this->state->get(self::STATER_KEY_FOR_KEY_VERSION, 1);
  }

}
