<?php

declare(strict_types=1);
namespace Drupal\sessionless\WebTokenAlgorithms;

use Jose\Component\Core\JWK;

final class WebTokenAlgorithmsCacheDecorator implements WebTokenAlgorithmsInterface {

  private array $cache;

  public function __construct(
    private WebTokenAlgorithmsInterface $decorated,
  ) {}

  public function getSignatureKey(): JWK {
    return $this->decorated->getSignatureKey();
  }

  public function getEncryptionKey(): JWK {
    return $this->decorated->getEncryptionKey();
  }

  public function sign(JWK $jwk, array $headers, array $payload): string {
    $cacheKey = implode('::', ['sign', json_encode($jwk), json_encode($headers), json_encode($payload)]);
    if (!isset($this->cache[$cacheKey])) {
      $this->cache[$cacheKey] = $this->decorated->sign($jwk, $headers, $payload);
    }
    return $this->cache[$cacheKey];
  }

  public function encrypt(JWK $jwk, array $headers, array $payload): string {
    $cacheKey = implode('::', ['encrypt', json_encode($jwk), json_encode($headers), json_encode($payload)]);
    if (!isset($this->cache[$cacheKey])) {
      $this->cache[$cacheKey] = $this->decorated->encrypt($jwk, $headers, $payload);
    }
    return $this->cache[$cacheKey];
  }

}
