<?php

namespace Drupal\sessionless\WebTokenAlgorithms;

use Jose\Component\Core\JWK;

/**
 * Crypto worker.
 *
 * Default algorithms copied over from Eclipse SmallRye, @link
 * https://quarkus.io/guides/security-jwt-build
 *
 * Links:
 * - The easy way: https://web-token.spomky-labs.com/v/v2.x/the-easy-way/easy
 * - SigAlgos:
 * https://web-token.spomky-labs.com/v/v2.x/the-components/signed-tokens-jws/signature-algorithms
 * - EncAlgos:
 * https://web-token.spomky-labs.com/v/v2.x/the-components/encrypted-tokens-jwe/encryption-algorithms
 * - Benchmarks:
 * https://web-token.spomky-labs.com/v/v2.x/benchmark/result-table
 * - RSA:
 * https://connect2id.com/products/nimbus-jose-jwt/examples/jwt-with-rsa-encryption
 * - Signing overview:
 * https://auth0.com/blog/json-web-token-signing-algorithms-overview/
 */
interface WebTokenAlgorithmsInterface {

  public function getSignatureKey(): JWK;

  public function sign(JWK $jwk, array $headers, array $payload): string;

  public function getEncryptionKey(): JWK;

  public function encrypt(JWK $jwk, array $headers, array $payload): string;

}
