<?php

namespace Drupal\sessionless\WebTokenService;

use Drupal\sessionless\KeyStorage\KeyStorageInterface;
use Drupal\sessionless\WebTokenAlgorithms\WebTokenAlgorithmsInterface;

/**
 * Sessionless encryption service.
 *
 * https://connect2id.com/products/nimbus-jose-jwt/examples/signed-and-encrypted-jwt
 */
final class WebTokenEncryptAndSignService implements WebTokenServiceInterface {

  use WebTokenServiceTrait;

  public function __construct(
    protected KeyStorageInterface $keyStorage,
    protected WebTokenAlgorithmsInterface $tokenAlgorithms,
  ) {}

  public function dump(mixed $data): string {
    $packed = $this->packData($data);
    $signed = $this->sign($packed);
    /** @noinspection PhpUnnecessaryLocalVariableInspection */
    $jwtString = $this->encrypt($signed);
    return $jwtString;
  }

  public function parse(string $token): mixed {
    try {
      $decrypted = $this->parseEncrypted($token);
      $packed = $this->parseSigned($decrypted);
      $data = $this->unpackData($packed);
      // @todo Catch only relevant exceptions.
    } catch (\Exception $e) {
      return NULL;
    }
    return $data;
  }

}
