<?php

namespace Drupal\sessionless\WebTokenService;

use Drupal\sessionless\KeyStorage\KeyStorageInterface;
use Drupal\sessionless\WebTokenAlgorithms\WebTokenAlgorithmsInterface;

/**
 * Only-signed-token service.
 *
 * Makes the token a bit smaller, but all payload data is public.
 */
final class WebTokenOnlySignService implements WebTokenServiceInterface {

  use WebTokenServiceTrait;

  public function __construct(
    protected KeyStorageInterface $keyStorage,
    protected WebTokenAlgorithmsInterface $tokenAlgorithms,
  ) {}

  public function dump(mixed $data): string {
    $packed = $this->packData($data);
    /** @noinspection PhpUnnecessaryLocalVariableInspection */
    $jwtString = $this->sign($packed);
    return $jwtString;
  }

  public function parse(string $token): mixed {
    try {
      $packed = $this->parseSigned($token);
      $data = $this->unpackData($packed);
      // @todo Catch only relevant exceptions.
    } catch (\Exception $e) {
      return NULL;
    }
    return $data;
  }

}
